package java.time.format;

import java.util.Collections;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.ResourceBundle;
import sun.util.locale.provider.LocaleProviderAdapter;
import java.util.AbstractMap;
import java.time.temporal.IsoFields;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Iterator;
import sun.util.locale.provider.CalendarDataUtility;
import java.time.chrono.JapaneseChronology;
import java.time.temporal.ChronoField;
import java.time.chrono.IsoChronology;
import java.time.chrono.Chronology;
import java.util.Comparator;
import java.util.Locale;
import java.time.temporal.TemporalField;
import java.util.Map;
import java.util.concurrent.ConcurrentMap;

class DateTimeTextProvider
{
    private static final ConcurrentMap<Map.Entry<TemporalField, Locale>, Object> CACHE;
    private static final Comparator<Map.Entry<String, Long>> COMPARATOR;
    
    DateTimeTextProvider() {
    }
    
    static DateTimeTextProvider getInstance() {
        return new DateTimeTextProvider();
    }
    
    public String getText(TemporalField temporalField, long n, TextStyle textStyle, Locale locale) {
        Object store = this.findStore(temporalField, locale);
        if ((store instanceof DateTimeTextProvider.LocaleStore)) {
            return ((DateTimeTextProvider.LocaleStore)store).getText(n, textStyle);
        }
        return null;
    }
    
    public String getText(Chronology chronology, TemporalField temporalField, long n, TextStyle textStyle, Locale locale) {
        if (chronology == IsoChronology.INSTANCE || !(temporalField instanceof ChronoField)) {
            return this.getText(temporalField, n, textStyle, locale);
        }
        int field;
        int value;
        if (temporalField == ChronoField.ERA) {
            field = 0;
            if (chronology == JapaneseChronology.INSTANCE) {
                if (n == -999L) {
                    value = 0;
                }
                else {
                    value = (int)n + 2;
                }
            }
            else {
                value = (int)n;
            }
        }
        else if (temporalField == ChronoField.MONTH_OF_YEAR) {
            field = 2;
            value = (int)n - 1;
        }
        else if (temporalField == ChronoField.DAY_OF_WEEK) {
            field = 7;
            value = (int)n + 1;
            if (value > 7) {
                value = 1;
            }
        }
        else if (temporalField == ChronoField.AMPM_OF_DAY) {
            field = 9;
            value = (int)n;
        }
        else {
            return null;
        }
        return CalendarDataUtility.retrieveJavaTimeFieldValueName(chronology.getCalendarType(), field, value, textStyle.toCalendarStyle(), locale);
    }
    
    public Iterator<Map.Entry<String, Long>> getTextIterator(TemporalField temporalField, TextStyle textStyle, Locale locale) {
        Object store = this.findStore(temporalField, locale);
        if ((store instanceof DateTimeTextProvider.LocaleStore)) {
            return ((DateTimeTextProvider.LocaleStore)store).getTextIterator(textStyle);
        }
        return null;
    }
    
    public Iterator<Map.Entry<String, Long>> getTextIterator(Chronology chronology, TemporalField temporalField, TextStyle textStyle, Locale locale) {
        if (chronology == IsoChronology.INSTANCE || !(temporalField instanceof ChronoField)) {
            return this.getTextIterator(temporalField, textStyle, locale);
        }
        int field;
        switch ((ChronoField)temporalField) {
            case ERA: {
                field = 0;
                break;
            }
            case MONTH_OF_YEAR: {
                field = 2;
                break;
            }
            case DAY_OF_WEEK: {
                field = 7;
                break;
            }
            case AMPM_OF_DAY: {
                field = 9;
                break;
            }
            default: {
                return null;
            }
        }
        Map<String, Integer> retrieveJavaTimeFieldValueNames = CalendarDataUtility.retrieveJavaTimeFieldValueNames(chronology.getCalendarType(), field, (textStyle == null) ? 0 : textStyle.toCalendarStyle(), locale);
        if (retrieveJavaTimeFieldValueNames == null) {
            return null;
        }
        ArrayList o = new ArrayList(retrieveJavaTimeFieldValueNames.size());
        switch (field) {
            case 0: {
                for (Map.Entry entry : retrieveJavaTimeFieldValueNames.entrySet()) {
                    int intValue = (int)entry.getValue();
                    if (chronology == JapaneseChronology.INSTANCE) {
                        if (intValue == 0) {
                            intValue = -999;
                        }
                        else {
                            intValue -= 2;
                        }
                    }
                    o.add(createEntry(entry.getKey(), Long.valueOf((long)intValue)));
                }
                break;
            }
            case 2: {
                for (Map.Entry entry2 : retrieveJavaTimeFieldValueNames.entrySet()) {
                    o.add(createEntry(entry2.getKey(), Long.valueOf((long)((int)entry2.getValue() + 1))));
                }
                break;
            }
            case 7: {
                for (Map.Entry entry3 : retrieveJavaTimeFieldValueNames.entrySet()) {
                    o.add(createEntry(entry3.getKey(), Long.valueOf((long)toWeekDay((int)entry3.getValue()))));
                }
                break;
            }
            default: {
                for (Map.Entry entry4 : retrieveJavaTimeFieldValueNames.entrySet()) {
                    o.add(createEntry(entry4.getKey(), Long.valueOf((long)(int)entry4.getValue())));
                }
                break;
            }
        }
        return o.iterator();
    }
    
    private Object findStore(TemporalField temporalField, Locale locale) {
        Map.Entry<TemporalField, Locale> entry = createEntry(temporalField, locale);
        Object o = DateTimeTextProvider.CACHE.get(entry);
        if (o == null) {
            DateTimeTextProvider.CACHE.putIfAbsent(entry, this.createStore(temporalField, locale));
            o = DateTimeTextProvider.CACHE.get(entry);
            return o;
        }
        return o;
    }
    
    private static int toWeekDay(int n) {
        if (n == 1) {
            return 7;
        }
        return n - 1;
    }
    
    private Object createStore(TemporalField temporalField, Locale locale) {
        HashMap<TextStyle, Object> hashMap = new HashMap();
        if (temporalField == ChronoField.ERA) {
            for (TextStyle textStyle4 : TextStyle.values()) {
                if (!textStyle4.isStandalone()) {
                    Map<String, Integer> retrieveJavaTimeFieldValueNames4 = CalendarDataUtility.retrieveJavaTimeFieldValueNames("gregory", 0, textStyle4.toCalendarStyle(), locale);
                    if (retrieveJavaTimeFieldValueNames4 != null) {
                        HashMap o5 = new HashMap();
                        for (Map.Entry entry4 : retrieveJavaTimeFieldValueNames4.entrySet()) {
                            o5.put(Long.valueOf((long)(int)entry4.getValue()), entry4.getKey());
                        }
                        if (!o5.isEmpty()) {
                            hashMap.put(textStyle4, o5);
                        }
                    }
                }
            }
            return new DateTimeTextProvider.LocaleStore(hashMap);
        }
        if (temporalField == ChronoField.MONTH_OF_YEAR) {
            for (TextStyle textStyle3 : TextStyle.values()) {
                Map<String, Integer> retrieveJavaTimeFieldValueNames3 = CalendarDataUtility.retrieveJavaTimeFieldValueNames("gregory", 2, textStyle3.toCalendarStyle(), locale);
                HashMap o4 = new HashMap();
                if (retrieveJavaTimeFieldValueNames3 != null) {
                    for (Map.Entry entry3 : retrieveJavaTimeFieldValueNames3.entrySet()) {
                        o4.put(Long.valueOf((long)((int)entry3.getValue() + 1)), entry3.getKey());
                    }
                }
                else {
                    int value2 = 0;
                    while (value2 <= 11) {
                        String retrieveJavaTimeFieldValueName2 = CalendarDataUtility.retrieveJavaTimeFieldValueName("gregory", 2, value2, textStyle3.toCalendarStyle(), locale);
                        if (retrieveJavaTimeFieldValueName2 == null) {
                            break;
                        }
                        o4.put(Long.valueOf((long)(value2 + 1)), retrieveJavaTimeFieldValueName2);
                        ++value2;
                    }
                }
                if (!o4.isEmpty()) {
                    hashMap.put(textStyle3, o4);
                }
            }
            return new DateTimeTextProvider.LocaleStore(hashMap);
        }
        if (temporalField == ChronoField.DAY_OF_WEEK) {
            for (TextStyle textStyle2 : TextStyle.values()) {
                Map<String, Integer> retrieveJavaTimeFieldValueNames2 = CalendarDataUtility.retrieveJavaTimeFieldValueNames("gregory", 7, textStyle2.toCalendarStyle(), locale);
                HashMap o3 = new HashMap();
                if (retrieveJavaTimeFieldValueNames2 != null) {
                    for (Map.Entry entry2 : retrieveJavaTimeFieldValueNames2.entrySet()) {
                        o3.put(Long.valueOf((long)toWeekDay((int)entry2.getValue())), entry2.getKey());
                    }
                }
                else {
                    int value = 1;
                    while (value <= 7) {
                        String retrieveJavaTimeFieldValueName = CalendarDataUtility.retrieveJavaTimeFieldValueName("gregory", 7, value, textStyle2.toCalendarStyle(), locale);
                        if (retrieveJavaTimeFieldValueName == null) {
                            break;
                        }
                        o3.put(Long.valueOf((long)toWeekDay(value)), retrieveJavaTimeFieldValueName);
                        ++value;
                    }
                }
                if (!o3.isEmpty()) {
                    hashMap.put(textStyle2, o3);
                }
            }
            return new DateTimeTextProvider.LocaleStore(hashMap);
        }
        if (temporalField == ChronoField.AMPM_OF_DAY) {
            for (TextStyle textStyle : TextStyle.values()) {
                if (!textStyle.isStandalone()) {
                    Map<String, Integer> retrieveJavaTimeFieldValueNames = CalendarDataUtility.retrieveJavaTimeFieldValueNames("gregory", 9, textStyle.toCalendarStyle(), locale);
                    if (retrieveJavaTimeFieldValueNames != null) {
                        HashMap o2 = new HashMap();
                        for (Map.Entry entry : retrieveJavaTimeFieldValueNames.entrySet()) {
                            o2.put(Long.valueOf((long)(int)entry.getValue()), entry.getKey());
                        }
                        if (!o2.isEmpty()) {
                            hashMap.put(textStyle, o2);
                        }
                    }
                }
            }
            return new DateTimeTextProvider.LocaleStore(hashMap);
        }
        if (temporalField == IsoFields.QUARTER_OF_YEAR) {
            String[] array = { "QuarterNames", "standalone.QuarterNames", "QuarterAbbreviations", "standalone.QuarterAbbreviations", "QuarterNarrows", "standalone.QuarterNarrows" };
            for (int i = 0; i < array.length; ++i) {
                String[] array2 = (String[])getLocalizedResource(array[i], locale);
                if (array2 != null) {
                    HashMap o = new HashMap();
                    for (int j = 0; j < array2.length; ++j) {
                        o.put(Long.valueOf((long)(j + 1)), array2[j]);
                    }
                    hashMap.put(TextStyle.values()[i], o);
                }
            }
            return new DateTimeTextProvider.LocaleStore(hashMap);
        }
        return "";
    }
    
    private static <A, B> Map.Entry<A, B> createEntry(A key, B value) {
        return new AbstractMap.SimpleImmutableEntry(key, value);
    }
    
    static <T> T getLocalizedResource(String s, Locale locale) {
        ResourceBundle javaTimeFormatData = LocaleProviderAdapter.getResourceBundleBased().getLocaleResources(locale).getJavaTimeFormatData();
        return javaTimeFormatData.containsKey(s) ? javaTimeFormatData.getObject(s) : null;
    }
    
    static Map.Entry access$000(Object o, Object o2) {
        return createEntry(o, o2);
    }
    
    static {
        CACHE = new ConcurrentHashMap(16, 0.75f, 2);
        COMPARATOR = new Comparator<Map.Entry<String, Long>>() {
            @Override
            public int compare(Map.Entry<String, Long> entry, Map.Entry<String, Long> entry2) {
                return ((String)entry2.getKey()).length() - ((String)entry.getKey()).length();
            }
        };
    }
    
    static final class LocaleStore
    {
        private final Map<TextStyle, Map<Long, String>> valueTextMap;
        private final Map<TextStyle, List<Map.Entry<String, Long>>> parsable;
        
        LocaleStore(Map<TextStyle, Map<Long, String>> valueTextMap) {
            this.valueTextMap = valueTextMap;
            HashMap<Object, ArrayList> parsable = new HashMap();
            ArrayList<Object> list = new ArrayList();
            for (Map.Entry entry : valueTextMap.entrySet()) {
                HashMap o = new HashMap();
                for (Map.Entry entry2 : ((Map)entry.getValue()).entrySet()) {
                    if (o.put(entry2.getValue(), DateTimeTextProvider.access$000(entry2.getValue(), entry2.getKey())) != null) {
                        continue;
                    }
                    continue;
                }
                ArrayList list2 = new ArrayList(o.values());
                Collections.sort(list2, DateTimeTextProvider.COMPARATOR);
                parsable.put(entry.getKey(), list2);
                list.addAll(list2);
                parsable.put(null, list);
            }
            Collections.sort(list, DateTimeTextProvider.COMPARATOR);
            this.parsable = parsable;
        }
        
        String getText(long l, TextStyle textStyle) {
            Map<Object, Object> map = (Map)this.valueTextMap.get(textStyle);
            return (map != null) ? ((String)map.get(Long.valueOf(l))) : null;
        }
        
        Iterator<Map.Entry<String, Long>> getTextIterator(TextStyle textStyle) {
            List<Object> list = (List)this.parsable.get(textStyle);
            return (list != null) ? list.iterator() : null;
        }
    }
}
